<?php
require_once('HT_Template.class.php');
require_once('HT_Ajax.class.php');

class HoneypotToolkit {
	var $absPath, $urlPath, $nonce, $pluginDirPath, $tpl, $ajax, $validationRegex;
	function __construct($mainFile) {
		$this->pluginDirPath = $this->get_plugin_dir();
		$this->absPath = $this->pluginDirPath . DIRECTORY_SEPARATOR . plugin_basename(dirname($mainFile));
		$this->urlPath = plugins_url("", $mainFile);
		$this->tpl = new HT_Template($this);
		$this->ajax = new HT_Ajax($this);
		$this->validationRegex = array(
			'new-ip-address-start'=>'/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/',
			'new-ip-address-end'=>'/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/',
			'new-ip-notes'=>'/^[\s|\S]{0,4000}$/',
			'edit-ip-address-start'=>'/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/',
			'edit-ip-address-end'=>'/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/',
			'edit-ip-notes'=>'/^[\s|\S]{0,4000}$/',
			'ht-honeypot-path'=>'/^(((https?):((\/\/)|(\\\\))|\/)+[\w\d:#\[\]\*@%\/;$()~_?\+-=\\\.&]*|)$/',
			'ht-ph-api-key'=>'/^[A-Za-z0-9]*$/',
			'ht-banned-usernames'=>'/^[A-Za-z0-9\s]*$/',
			'default'=>'/^.*$/'
		);
		$this->add_hooks($mainFile);
	}

	function get_plugin_dir() {
		return preg_replace('/\\' . DIRECTORY_SEPARATOR . 'honeypot-toolkit\\' . DIRECTORY_SEPARATOR . 'lib$/', '', dirname(__FILE__));
	}
	
	function add_hooks($mainFile) {
		register_activation_hook($mainFile,array($this, 'activate'));
		if (get_option('ht_use_project_honeypot') == '1' && get_option('ht_honeypot_path') != '') {
			if (get_option('ht_use_custom_honeypot') == '1') {
				$maxPos = 3;
			} else {
				$maxPos = 2;
			}
			switch(mt_rand(1,$maxPos)) {
				case 1:
					add_filter('wp_nav_menu', array($this, 'insert_honeypot_link'), 99, 1);
					add_filter('wp_page_menu', array($this, 'insert_honeypot_link'), 99, 1);
					break;
				case 2:
					add_action('wp_footer', array($this, 'print_honeypot_link'), 99, 1);
					break;
				case 3:
					add_action('ht_custom_honeypot', array($this, 'print_honeypot_link'));
					break;
			}
		}

		add_filter('init', array($this, 'init'));

		add_action('admin_init', array($this, 'register_admin_style'));
		add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_style'));
		add_action('admin_menu', array($this, 'admin_menu'), 9);
		add_action('admin_notices', array($this, 'add_ajax_notices'));
		
		add_action('shutdown', array($this, 'check_for_404'));
		add_filter('authenticate', array($this, 'check_banned_users'), 1, 2);
		add_action('wp_login_failed', array($this, 'login_failed'), 1, 1);
		add_action('wp_login', array($this, 'user_logged_in'));
		add_filter('wp_login_errors', array($this, 'show_bad_login_count'), 10, 1);
		
		//Ajax functions
		add_action('wp_ajax_HT_blacklist_ip', array($this->ajax, 'blacklist_ip'));
		add_action('wp_ajax_HT_whitelist_ip', array($this->ajax, 'whitelist_ip'));
		add_action('wp_ajax_HT_save_settings', array($this->ajax, 'save_settings'));
		add_action('wp_ajax_HT_remove_ip', array($this->ajax, 'remove_ip'));
		add_action('wp_ajax_HT_retrieve_entry_details', array($this->ajax, 'retrieve_entry_details'));
		add_action('wp_ajax_HT_edit_ip_entry', array($this->ajax, 'edit_ip_list_entry'));
		add_action('wp_ajax_HT_retrieve_whitelist', array($this->ajax, 'retrieve_whitelist'));
		add_action('wp_ajax_HT_retrieve_blocked_list', array($this->ajax, 'retrieve_blocked_list'));
		add_action('wp_ajax_HT_retrieve_activity_list', array($this->ajax, 'retrieve_activity_list'));
		add_action('wp_ajax_HT_retrieve_page_limit', array($this->ajax, 'retrieve_page_limit'));
	}

	function init() {
		##Create nonce
		$this->nonce = wp_create_nonce(plugin_basename(__FILE__));
	}

	function verify_nonce($nonce) {
		return wp_verify_nonce( $nonce, plugin_basename(__FILE__) );
	}
	
	function check_for_404() {
		global $wpdb;
		$remoteAddress = sanitize_text_field($_SERVER['REMOTE_ADDR']);
		$ipNumber = HoneypotToolkit::calculate_ip_number($remoteAddress);
		if (!HoneypotToolkit::check_whitelist($ipNumber)) {
			if (is_404() && get_option('ht_404_mon', "0") == "1") {
				$ht404Limit = get_option('ht_404_limit', "10");
				$ht404TimeSpan = get_option('ht_404_time_span', "3600");
				
				$deleteActivityQuery = "DELETE FROM ".$wpdb->prefix."ht_activity WHERE last_activity < %d AND activity_type='404'";
				$wpdb->query($wpdb->prepare($deleteActivityQuery, time()-$ht404TimeSpan));
				
				$storedActivity = $wpdb->get_row($wpdb->prepare("SELECT activity_id, activity_count, notes FROM ".$wpdb->prefix."ht_activity WHERE ip_address=%s AND activity_type='404'", $remoteAddress), ARRAY_A);
				$currentCount = 0;
				if (is_array($storedActivity)) {
					$activityNotes = $storedActivity['notes']."<br />".htmlentities($_SERVER['REQUEST_URI']);
					$wpdb->update($wpdb->prefix."ht_activity", array("activity_count"=>$storedActivity['activity_count']+1, "last_activity"=>time(), "notes"=>$activityNotes), array("activity_id"=>$storedActivity['activity_id'], "activity_type"=>"404"));
					$currentCount = $storedActivity['activity_count']+1;
				} else {
					$activityNotes = htmlentities($_SERVER['REQUEST_URI']);
					$wpdb->insert($wpdb->prefix."ht_activity", array("ip_address"=>$remoteAddress, "activity_count"=>1, "activity_type"=>"404", "last_activity"=>time(), "notes"=>$activityNotes));
					$currentCount = 1;
				}

				if ($currentCount >= $ht404Limit) {
					$wpdb->insert($wpdb->prefix."ht_ip_list", array("ip_address_start"=>$remoteAddress, "ip_address_end"=>$remoteAddress, "ip_number_start"=>$ipNumber, "ip_number_end"=>$ipNumber, "offense_level"=>1, "insert_time"=>time(), "notes"=>$activityNotes, "activity_count"=>1));
					$deleteActivityQuery = "DELETE FROM ".$wpdb->prefix."ht_activity WHERE ip_address=%s";
					$wpdb->query($wpdb->prepare($deleteActivityQuery, $remoteAddress));
				}
			}
		}

		$this->remove_blocked_ips();
	}
	
	function remove_blocked_ips() {
		global $wpdb;
		##Check for blocked ips to be removed
		if (false === get_site_transient('HT_delete_blocked_ips')) {
			$ht404BlockTime = get_option('ht_404_block_time', "86400");
			$htLoginBlockTime = get_option('ht_login_block_time', "86400");
				
			$deleteBlockedIPQuery = "DELETE FROM ".$wpdb->prefix."ht_ip_list WHERE (insert_time <= %d AND offense_level = 1) OR (insert_time <= %d AND offense_level = 10)";
			$wpdb->query($wpdb->prepare($deleteBlockedIPQuery, time()-$ht404BlockTime, time()-$htLoginBlockTime));
			set_site_transient('HT_delete_blocked_ips', 1, 1800);
		}

		##Check Project Honeypot and Spamcop records
		if (false === get_site_transient('HT_spam_records_checked')) {
			$htPHAPIKey = get_option('ht_ph_api_key');
			$htPHMaxDays = get_option('ht_ph_bl_max_days', '255');
			$htPHThreatLevel = get_option('ht_ph_bl_threat_score', '10');
			$htPHCheckIPInterval = get_option('ht_ph_check_ip_interval', '14');
			$htUseProjectHoneypot = get_option('ht_use_project_honeypot', '0');
			$htUseSpamcop = get_option('ht_use_spamcop', '0');

			if ($htUseProjectHoneypot == '1') {
				$honeypotRecordQuery = "SELECT ip_id, ip_address_start FROM ".$wpdb->prefix."ht_ip_list WHERE offense_level = 11";
				$allHoneypotRecords = $wpdb->get_results($honeypotRecordQuery, ARRAY_A);
				foreach($allHoneypotRecords as $honeypotRecord) {
					$recordUpdated == 0;
					$dnsData = @dns_get_record($htPHAPIKey . "." . implode(".", array_reverse(explode(".", $honeypotRecord['ip_address_start']))) . ".dnsbl.httpbl.org");
					if (is_array($dnsData)) {
						foreach($dnsData as $dnsRecord) {
							if (isset($dnsRecord['type']) && $dnsRecord['type'] == 'A') {
								if (isset($dnsRecord['ip'])) {
									$ipThreatScore = explode(".", $dnsRecord['ip']);
									if ($ipThreatScore[1] <= $htPHMaxDays && $ipThreatScore[2] >= $htPHThreatLevel && $ipThreatScore[3] > 0) {
										$wpdb->update($wpdb->prefix."ht_ip_list", array("insert_time"=>time(), "notes"=>"PROJECT HONEYPOT: ".$dnsRecord['ip']), array("ip_id"=>$honeypotRecord['ip_id']));
										$recordUpdated = 1;
									}
								}
							}
						}
					}

					if ($recordUpdated == 0) {
						$deleteSpamRecordQuery = "DELETE FROM ".$wpdb->prefix."ht_activity WHERE ip_id=%d";
						$wpdb->query($wpdb->prepare($deleteSpamRecordQuery, $honeypotRecord['ip_id']));
					}
				}
			}

			if ($htUseSpamcop == '1') {
				$spamcopRecordQuery = "SELECT ip_id, ip_address_start FROM ".$wpdb->prefix."ht_ip_list WHERE offense_level = 12";
				$allSpamcopRecords = $wpdb->get_results($spamcopRecordQuery, ARRAY_A);
				foreach($allSpamcopRecords as $spamcopRecord) {
					$recordUpdated == 0;
					$dnsData = @dns_get_record(implode(".", array_reverse(explode(".", $spamcopRecord['ip_address_start']))) . ".bl.spamcop.net");
					if (is_array($dnsData)) {
						foreach($dnsData as $dnsRecord) {
							if (isset($dnsRecord['type']) && $dnsRecord['type'] == 'A' && $dnsRecord['ip'] == '127.0.0.2') {
								$wpdb->update($wpdb->prefix."ht_ip_list", array("insert_time"=>time(), "notes"=>"SPAMCOP ENTRY"), array("ip_id"=>$spamcopRecord['ip_id']));
								$recordUpdated = 1;
							}
						}
					}

					if ($recordUpdated == 0) {
						$deleteSpamRecordQuery = "DELETE FROM ".$wpdb->prefix."ht_activity WHERE ip_id=%d";
						$wpdb->query($wpdb->prepare($deleteSpamRecordQuery, $spamcopRecord['ip_id']));
					}
				}
			}
			set_site_transient('HT_spam_records_checked', 1, ($htPHCheckIPInterval * 86400));
		}
	}
	
	function show_bad_login_count($errors) {
		global $wpdb;
		if (get_option('ht_show_login_count') == '1') {
			$htLoginLimit = get_option('ht_login_limit', "3");
			$activityCount = $wpdb->get_row($wpdb->prepare("SELECT activity_id, activity_count FROM ".$wpdb->prefix."ht_activity WHERE ip_address=%s AND activity_type='login'", $_SERVER['REMOTE_ADDR']), ARRAY_A);
			if (is_array($activityCount)) {
				$errors->add('ht_login_attempts', '<strong>Login attempts left:</strong> '.($htLoginLimit-$activityCount['activity_count']));
			} else {
				$errors->add('ht_login_attempts', '<strong>Login attempts left:</strong> '.$htLoginLimit);
			}
		}
		return $errors;
	}
	
	function check_banned_users($user, $username) {
		global $wpdb;
		$remoteAddress = sanitize_text_field($_SERVER['REMOTE_ADDR']);

		$ipNumber = $this->calculate_ip_number($remoteAddress);
		if (!$this->check_whitelist($ipNumber) && get_option('ht_login_mon', "0") == "1") {
			$bannedUsernames = get_option('ht_banned_usernames', array());

			if (in_array($username, $bannedUsernames)) {
				$wpdb->insert($wpdb->prefix."ht_ip_list", array("ip_address_start"=>$remoteAddress, "ip_address_end"=>$remoteAddress, "ip_number_start"=>$ipNumber, "ip_number_end"=>$ipNumber, "offense_level"=>10, "insert_time"=>time(), "notes"=>htmlentities(strip_tags($username)), "activity_count"=>1));
				$deleteActivityQuery = "DELETE FROM ".$wpdb->prefix."ht_activity WHERE ip_address=%s";
				$wpdb->query($wpdb->prepare($deleteActivityQuery, $remoteAddress));
				$htResponseCode = get_option('ht_response_code', "503");
				http_response_code($htResponseCode);
				die();
			}
		}
		return $user;
	}
	
	function login_failed($username) {
		global $wpdb;
		$remoteAddress = sanitize_text_field($_SERVER['REMOTE_ADDR']);
		$ipNumber = $this->calculate_ip_number($remoteAddress);
		if ($ipNumber > 0 && !$this->check_whitelist($ipNumber) && get_option('ht_login_mon', "0") == "1") {
			$htLoginLimit = get_option('ht_login_limit', "3");
			$htLoginTimeSpan = get_option('ht_login_time_span', "86400");
			
			$activityNotes = htmlentities(strip_tags($username));
			$deleteActivityQuery = "DELETE FROM ".$wpdb->prefix."ht_activity WHERE last_activity < %d AND activity_type='login'";
			$wpdb->query($wpdb->prepare($deleteActivityQuery, time()-$htLoginTimeSpan));

			$storedActivity = $wpdb->get_row($wpdb->prepare("SELECT activity_id, activity_count, notes FROM ".$wpdb->prefix."ht_activity WHERE ip_address=%s AND activity_type='login'", $remoteAddress), ARRAY_A);
			$currentCount = 0;
			if (is_array($storedActivity)) {
				$activityNotes = $storedActivity['notes'].'<br />'.$activityNotes;
				$wpdb->update($wpdb->prefix."ht_activity", array("activity_count"=>$storedActivity['activity_count']+1, "last_activity"=>time()), array("activity_id"=>$storedActivity['activity_id'], "activity_type"=>"login", "notes"=>$activityNotes));
				$currentCount = $storedActivity['activity_count']+1;
			} else {
				$wpdb->insert($wpdb->prefix."ht_activity", array("ip_address"=>$remoteAddress, "activity_count"=>1, "activity_type"=>"login", "last_activity"=>time(), "notes"=>$activityNotes));
				$currentCount = 1;
			}

			if ($currentCount >= $htLoginLimit) {
				$wpdb->insert($wpdb->prefix."ht_ip_list", array("ip_address_start"=>$remoteAddress, "ip_address_end"=>$remoteAddress, "ip_number_start"=>$ipNumber, "ip_number_end"=>$ipNumber, "offense_level"=>10, "insert_time"=>time(), "notes"=>$activityNotes, "activity_count"=>$currentCount));
				$deleteActivityQuery = "DELETE FROM ".$wpdb->prefix."ht_activity WHERE ip_address=%s";
				$wpdb->query($wpdb->prepare($deleteActivityQuery, $remoteAddress));
				$htResponseCode = get_option('ht_response_code', "503");
				http_response_code($htResponseCode);
				die();
			}
		}
	}

	function user_logged_in() {
		global $wpdb;
		$deleteActivityQuery = "DELETE FROM ".$wpdb->prefix."ht_activity WHERE ip_address = %s AND activity_type='login'";
		$wpdb->query($wpdb->prepare($deleteActivityQuery, sanitize_text_field($_SERVER['REMOTE_ADDR'])));
	}
	
	
	function add_ajax_notices() {
		?>
		<div id="HT-ui-notices" title="Basic dialog">
			<div id="HT-ajax-notices-container">
			</div>
		</div>
		<?php
	}
		
	function register_admin_style() {
		wp_register_style('font-awesome-4.7.0', '//maxcdn.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css');
		wp_register_style('HT-admin', $this->urlPath . '/css/admin.css');
	}
	
	function enqueue_admin_style() {
		wp_enqueue_style('font-awesome-4.7.0');
		wp_enqueue_style('HT-admin');
	}
	
	function admin_menu() {
		$pluginPage=add_menu_page( 'Honeypot Toolkit', 'Honeypot Toolkit', 'administrator', 'Honeypot_Protector', array($this->tpl, 'settings_page'), $this->urlPath.'/img/honeypot-icon.png');
		add_action('admin_head-'.$pluginPage, array($this->tpl, 'common_js'));
		add_action('admin_head-'.$pluginPage, array($this->tpl, 'settings_page_js'));
		
		$pluginPage=add_submenu_page('Honeypot_Protector', 'Settings', 'Settings', 'administrator', 'Honeypot_Protector', array($this->tpl, 'settings_page'));
		add_action('admin_head-'.$pluginPage, array($this->tpl, 'common_js'));
		add_action('admin_head-'.$pluginPage, array($this->tpl, 'settings_page_js'));
		
		$pluginPage=add_submenu_page('Honeypot_Protector', 'Blocked List', 'Blocked List', 'administrator', 'HT_blocked_list', array($this->tpl, 'blocked_list_page'));
		add_action('admin_head-'.$pluginPage, array($this->tpl, 'common_js'));
		add_action('admin_head-'.$pluginPage, array($this->tpl, 'blocked_list_page_js'));
		
		$pluginPage=add_submenu_page('Honeypot_Protector', 'Activity List', 'Activity List', 'administrator', 'HT_activity_list', array($this->tpl, 'activity_list_page'));
		add_action('admin_head-'.$pluginPage, array($this->tpl, 'common_js'));
		add_action('admin_head-'.$pluginPage, array($this->tpl, 'activity_list_page_js'));
		
		$pluginPage=add_submenu_page('Honeypot_Protector', 'Whitelist', 'Whitelist', 'administrator', 'HT_whitelist', array($this->tpl, 'whitelist_page'));
		add_action('admin_head-'.$pluginPage, array($this->tpl, 'common_js'));
		add_action('admin_head-'.$pluginPage, array($this->tpl, 'whitelist_page_js'));
	}
	
	function activate() {
		global $wpdb;
		$IPTableSQL = "CREATE TABLE ".$wpdb->prefix."ht_ip_list (
			ip_id bigint(20) UNSIGNED NOT NULL AUTO_INCREMENT,
			ip_address_start varchar(15) NOT NULL DEFAULT '',
			ip_address_end varchar(15) NOT NULL DEFAULT '',
			ip_number_start bigint(20) NOT NULL DEFAULT 0,
			ip_number_end bigint(20) NOT NULL DEFAULT 0,
			insert_time int(12) NOT NULL DEFAULT 0,
			offense_level tinyint UNSIGNED NOT NULL DEFAULT 1,
			notes longtext NOT NULL DEFAULT '',
			activity_count bigint(20) NOT NULL DEFAULT 0,
			PRIMARY KEY (ip_id),
			KEY ht_address_idx (ip_number_start, ip_number_end, insert_time, offense_level)
		);";
		if($wpdb->get_var("SHOW TABLES LIKE '".$wpdb->prefix."ht_ip_list'") != $wpdb->prefix."ht_ip_list") {
			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($IPTableSQL);
		}

		$activityTableSQL = "CREATE TABLE ".$wpdb->prefix."ht_activity (
			activity_id bigint(20) UNSIGNED NOT NULL AUTO_INCREMENT,
			ip_address varchar(15) NOT NULL DEFAULT '',
			last_activity int(12) NOT NULL DEFAULT 0,
			activity_count int(3) NOT NULL DEFAULT 1,
			activity_type varchar(25) NOT NULL DEFAULT '',
			notes longtext NOT NULL default '',
			PRIMARY KEY (activity_id),
			KEY ht_activity_idx (ip_address, activity_type)
		);";
		if($wpdb->get_var("SHOW TABLES LIKE '".$wpdb->prefix."ht_activity'") != $wpdb->prefix."ht_activity") {
			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($activityTableSQL);
		}
		
		if (get_option('ht_only_allow_whitelist') == "") {
			update_option('ht_only_allow_whitelist', "0");
		}

		if (get_option('ht_login_mon') == "") {
			update_option('ht_login_mon', "0");
		}
		
		if (get_option('ht_login_limit') == "") {
			update_option('ht_login_limit', "3");
		}

		if (get_option('ht_login_time_span') == "") {
			update_option('ht_login_time_span', "3600");
		}

		if (get_option('ht_login_block_time') == "") {
			update_option('ht_login_block_time', "86400");
		}

		if (get_option('ht_show_login_count') == "") {
			update_option('ht_show_login_count', "0");
		}

		if (get_option('ht_404_mon') == "") {
			update_option('ht_404_mon', "0");
		}
		
		if (get_option('ht_404_limit') == "") {
			update_option('ht_404_limit', "10");
		}

		if (get_option('ht_404_time_span') == "") {
			update_option('ht_404_time_span', "3600");
		}

		if (get_option('ht_404_block_time') == "") {
			update_option('ht_404_block_time', "86400");
		}

		if (get_option('ht_response_code') == "") {
			update_option('ht_response_code', "503");
		}

		if (get_option('ht_ph_bl_max_days') == "") {
			update_option('ht_ph_bl_max_days', "255");
		}

		if (get_option('ht_ph_bl_threat_score') == "") {
			update_option('ht_ph_bl_threat_score', "10");
		}

		if (get_option('ht_use_project_honeypot') == "") {
			update_option('ht_use_project_honeypot', "0");
		}

		if (get_option('ht_use_spamcop') == "") {
			update_option('ht_use_spamcop', "0");
		}
	}
	
	function gen_honeypot_link() {
		$honeypotLink = '';
		$honeypotPath = get_option('ht_honeypot_path', '/');
		$linkText = $this->gen_random_string(mt_rand(5,32));
		switch(mt_rand(1,5)) {
			default:
			case 1:
				$honeypotLink = '<div style="display: none;"><a href="'.$honeypotPath.'" title="'.$linkText.'">'.$linkText.'</a></div>';
				break;
			case 2:
				$honeypotLink = '<a href="'.$honeypotPath.'" style="display: none;" title="'.$linkText.'">'.$linkText.'</a>';
				break;
			case 3:
				$honeypotLink = '<a href="'.$honeypotPath.'" title="'.$linkText.'"><!-- '.$linkText.' --></a>';
				break;
			case 4:
				$honeypotLink = '<!-- <a href="'.$honeypotPath.'" title="'.$linkText.'">'.$linkText.'</a> -->';
				break;
			case 5:
				$honeypotLink = '<a href="'.$honeypotPath.'" title="'.$linkText.'"></a>';
				break;
		}
		return $honeypotLink;
	}

	function gen_random_string($length) {
		$string = '';
		for ($i=0; $i<$length; $i++) {
			$char = '';
			switch(mt_rand(1,5)) {
				case 2:
					$char = chr(mt_rand(65,79));
					break;
				case 4:
					$char = chr(mt_rand(80,90));
					break;
				case 1:
					$char = chr(mt_rand(109,122));
					break;
				case 3:
					$char = chr(mt_rand(97,108));
					break;
				case 5:
					$char = ' ';
					break;
			}
			$string .= $char;
		}
		return $string;
	}
	
	function insert_honeypot_link($navMenuHTML) {
		return $navMenuHTML . $this->gen_honeypot_link();
	}

	function print_honeypot_link() {
		print $this->gen_honeypot_link();
	}

	public static function check_whitelist($ipNumber) {
		global $wpdb;
		if ($wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM ".$wpdb->prefix."ht_ip_list WHERE ip_number_start <= %d AND ip_number_end >= %d AND offense_level = 0", $ipNumber, $ipNumber)) > 0) {
			if (!isset($GLOBALS['ht_whitelist_count_incremented'])) {
				$incrementActivityQuery = "UPDATE ".$wpdb->prefix."ht_ip_list SET activity_count=activity_count+1 WHERE ip_number_start <= %d AND ip_number_end >= %d";
				$incrementActivityRes = $wpdb->query($wpdb->prepare($incrementActivityQuery, $ipNumber, $ipNumber));
				$GLOBALS['ht_whitelist_count_incremented'] = 1;
			}
			return true;
		}
		return false;
	}
	
	public static function check_block_list() {
		global $wpdb;
		$ht404BlockTime = get_option('ht_404_block_time', "86400");
		$htLoginBlockTime = get_option('ht_login_block_time', "86400");
		$htOnlyAllowWhitelist = get_option('ht_only_allow_whitelist', '0');
		$htResponseCode = get_option('ht_response_code', "503");
		$htPHAPIKey = get_option('ht_ph_api_key');
		$htPHMaxDays = get_option('ht_ph_bl_max_days', '255');
		$htPHThreatLevel = get_option('ht_ph_bl_threat_score', '10');
		$htUseProjectHoneypot = get_option('ht_use_project_honeypot', '0');
		$htUseSpamcop = get_option('ht_use_spamcop', '0');

		foreach(array('HTTP_X_FORWARDED_FOR', 'REMOTE_ADDR') as $serverKey) {
			if (isset($_SERVER[$serverKey])) {
				$serverKeyVal = sanitize_text_field($_SERVER[$serverKey]);
				$ipNumber = HoneypotToolkit::calculate_ip_number($serverKeyVal);
				if (!HoneypotToolkit::check_whitelist($ipNumber)) {
					if ($htOnlyAllowWhitelist == '1' || $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM ".$wpdb->prefix."ht_ip_list WHERE ip_number_start <= %d AND ip_number_end >= %d AND (offense_level >= 11 OR (insert_time > %d AND offense_level = 1) OR (insert_time > %d AND offense_level = 10))", $ipNumber, $ipNumber, time()-$ht404BlockTime, time()-$htLoginBlockTime)) > 0) {
						$incrementActivityQuery = "UPDATE ".$wpdb->prefix."ht_ip_list SET insert_time=".time().", activity_count=activity_count+1 WHERE ip_number_start <= %d AND ip_number_end >= %d";
						$incrementActivityRes = $wpdb->query($wpdb->prepare($incrementActivityQuery, $ipNumber, $ipNumber));
						http_response_code($htResponseCode);
						die();
					} else {
						if ($htUseProjectHoneypot == '1' && $htPHAPIKey != '') {
							$dnsData = @dns_get_record($htPHAPIKey . "." . implode(".", array_reverse(explode(".", $serverKeyVal))) . ".dnsbl.httpbl.org");
							if (is_array($dnsData)) {
								foreach($dnsData as $dnsRecord) {
									if (isset($dnsRecord['type']) && $dnsRecord['type'] == 'A') {
										if (isset($dnsRecord['ip'])) {
											$ipThreatScore = explode(".", $dnsRecord['ip']);
											if ($ipThreatScore[1] <= $htPHMaxDays && $ipThreatScore[2] >= $htPHThreatLevel && $ipThreatScore[3] > 0) {
												$wpdb->insert($wpdb->prefix."ht_ip_list", array("ip_address_start"=>$serverKeyVal, "ip_address_end"=>$serverKeyVal, "ip_number_start"=>$ipNumber, "ip_number_end"=>$ipNumber, "offense_level"=>11, "insert_time"=>time(), "notes"=>"PROJECT HONEYPOT: ".$dnsRecord['ip'], "activity_count"=>1));
												$deleteActivityQuery = "DELETE FROM ".$wpdb->prefix."ht_activity WHERE ip_address=%s";
												$wpdb->query($wpdb->prepare($deleteActivityQuery, $serverKeyVal));
												http_response_code($htResponseCode);
												die();
											}
										}
									}
								}
							}
						}

						if ($htUseSpamcop == '1') {
							$dnsData = @dns_get_record(implode(".", array_reverse(explode(".", $serverKeyVal))) . ".bl.spamcop.net");
							if (is_array($dnsData)) {
								foreach($dnsData as $dnsRecord) {
									if (isset($dnsRecord['type']) && $dnsRecord['type'] == 'A' && $dnsRecord['ip'] == '127.0.0.2') {
										$wpdb->insert($wpdb->prefix."ht_ip_list", array("ip_address_start"=>$serverKeyVal, "ip_address_end"=>$serverKeyVal, "ip_number_start"=>$ipNumber, "ip_number_end"=>$ipNumber, "offense_level"=>12, "insert_time"=>time(), "notes"=>"SPAMCOP ENTRY", "activity_count"=>1));
										$deleteActivityQuery = "DELETE FROM ".$wpdb->prefix."ht_activity WHERE ip_address=%s";
										$wpdb->query($wpdb->prepare($deleteActivityQuery, $serverKeyVal));
										http_response_code($htResponseCode);
										die();
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	public static function calculate_ip_number($ipAddress) {
		if (preg_match('/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/', $ipAddress)) {
			return ip2long($ipAddress);
		} else {
			return 0;
		}
	}

}
?>